/* DChub - a Direct Connect hub clone for Linux
 * Copyright (C) 2001 Eric Prevoteau
 *
 * ClientBot.c: Copyright (C) Dirk Rieger
 *
 * This progrmp is free software; you can redistribute it and/or modify
 * it under the temps of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This progrmp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the mpplied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this progrmp; if not, write to the Free Software
 * Foundation, Inc., 59 Tmpple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <glib.h>
#include <pthread.h>
/* includes for incoming_tcp */

#include <errno.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/file.h> 
#include <ctype.h>
#include <fcntl.h>

/* end includes for incoming_tcp */

#include "dc_com.h"			/* we use the same function as the hub one to communicate */
#include "extprog_toolkit.h"
#include "network.h"



#ifndef MSG_NOSIGNAL
#define HAVE_NO_MSG_NOSIGNAL 1
#define MSG_NOSIGNAL 0
#endif



int hub_com;			/* socket fd to use to communicate with hub */
G_LOCK_DEFINE(hub_com);		/* to avoid conflict between the main function and threads */

typedef enum
{
	UNDEFINED=0,
	ACTIVE,
	PASSIVE
} UCNX;

typedef struct
{
	UCNX type;                  /* what type of connection the remote user has */
	char *addr;                 /* address of the remote user (NULL for UNDEFINED and PASSIVE, !=NULL for ACTIVE) */
	GString *client;
	GString *version;
	GString *lock;
	GString *pk;
	int has_bzlist;
	int got_lock;
	int got_myinfo;
	GString *d_client;
	GString *d_version;
} XTRA_INFO;

/* userlist handling */
G_LOCK_DEFINE(hub_user_list);
GPtrArray *hub_user_list=NULL;
GPtrArray *hub_op_list=NULL;
GPtrArray *hub_user_xtra_info=NULL;
GPtrArray *hub_user_uinfo=NULL;

/* definitions for incoming_tcp */

/**************************************************************************************/
/* due to the fact all the following data are used by threads, they must be protected */
/**************************************************************************************/

/* this is the lists of all running connections */
G_LOCK_DEFINE(waiting_action);                          /* this mutex protects waiting_action array */
GPtrArray *waiting_action=NULL;                         /* array of pointers on WAIT_ACT structure */

typedef enum
{
	CNX_ON_CONNECT,                 /* created after a connect (in reply to a $ConnectToMe) */
	CNX_ON_ACCEPT,                          /* created after accepting an incoming connection */
} CNX_TYPE;

typedef struct
{
	int running;                                    /* 0=not running, 1=waiting remote connection, 2=running, 3=killed */
	CNX_TYPE cnx_type;                      /* thread created on accept or connect ? */
	time_t last_touch;                      /* date of the last change of this task */
	pthread_t thread_id;                    /* !=0, this is the thread id */
	int sock_fd;                                    /* socket descriptor used by the thread */
	GString *remote_nick;           /* nick of the user on the remote side */
	GString *remote_addr;           /* addr:port of the remote side */
	struct wait_revcon *run_task;   /* task associated to this thread */
	GString *disp_info;                     /* info displayed when the xfer list is requested */

	GStringChunk *cap_str;          /* used to store remote client capabilities */
	GPtrArray *cap_ptr;                     /* array of pointers on remote client capabilities */
} WAIT_ACT;

int in_sck=-1;
G_LOCK_DEFINE(in_sck);
char my_incom[8192];

#define max(a,b)     (((a)>(b))?(a):(b))

char *lockstart[]={     "EXTENDEDPROTOCOL",             /* Default dc++ loststart */
                        "AAAAAAAAAAAAAAAA",             /* dc++k hiding with disabled extended protocol */
                        NULL};

char *pkstart[]={       "DCPLUSPLUS",                   /* Default dc++ pkstart */
                        "BBBBBBBBBB",                   /* dc++k hiding with disabled extended protocol */
                        "mldonkey",                     /* edonkey-client searching in dc-hubs as well as faker with 9.31GB share */
                        NULL};

char *pkend[]={         "ABCABC",                       /* Default dc++ pkend */
                        NULL};

/* end definitions for incoming_tcp */

char *my_name;
gchar *my_hostip=NULL;		/* it is the address to use to perform search or download */

time_t gl_cur_time;
/**************************************************/
/* this is a list of search to periodically start */
/**************************************************/
typedef struct
{
	time_t last_global_scan;
	gchar *search_result_return_address;		/* it is my_hostip my_port */
	GString *search_query;				/* it is the constant part of the search (file type, search type, search pattern) */
	unsigned int on_start:1;			/* (not yet supported) ==1, when this entry is added, a search is immediatly started */
	unsigned int on_incoming:1;			/* ==1, when a new user enters, a "one user" search is started */
	unsigned int global_periodic:1;			/* (not yet supported) ==1, the scan is periodically started */
} AUTOSEARCH;

GArray *search_to_start=NULL;
G_LOCK_DEFINE(search_to_start);

/********************************/
/* free 1 user xtra information */
/********************************/
static void free_xtra_info(XTRA_INFO *xi)
{
	if(xi==NULL)
		return;
	if(xi->addr!=NULL)
                free(xi->addr);
	if(xi->client->str!=NULL)
		g_string_free(xi->client,TRUE);
	if(xi->version->str!=NULL)
		g_string_free(xi->version,TRUE);
	if(xi->d_client->str!=NULL)
		g_string_free(xi->d_client,TRUE);
	if(xi->d_version->str!=NULL)
		g_string_free(xi->d_version,TRUE);
	if(xi->lock->str!=NULL)
		g_string_free(xi->lock,TRUE);
	if(xi->pk->str!=NULL)
		g_string_free(xi->pk,TRUE);
	free(xi);
#if 0
	printf("free xtra info finished\n");
#endif
}

/*********************************/
/* empty the hub_user_list array */
/*********************************/
void reset_hub_user_list(void)
{
	int i;
	char *p;

	G_LOCK(hub_user_list);
	/* reset hub user list */
	if(hub_user_list!=NULL) 
	{
#if 0
		printf("resetting hub_user_list\n");
#endif
		for(i=0;i<hub_user_list->len;i++)
		{
			p=g_ptr_array_index(hub_user_list,i);
			if(p)
				free(p);
		}
		g_ptr_array_free(hub_user_list,TRUE);
	}
	hub_user_list=g_ptr_array_new();

	/* reset hub user uinfo */
	if(hub_user_uinfo!=NULL)
	{
#if 0
		printf("resetting hub_user_uinfo\n");
#endif
		for(i=0;i<hub_user_uinfo->len;i++)
		{
			p=g_ptr_array_index(hub_user_uinfo,i);
			if(p)
				free(p);
		}
		g_ptr_array_free(hub_user_uinfo,TRUE);
	}
	hub_user_uinfo=g_ptr_array_new();

	/* reset hub op list */
	if(hub_op_list!=NULL)
	{
#if 0
		printf("resetting hub_op_list\n");
#endif
		for(i=0;i<hub_op_list->len;i++)
		{
			p=g_ptr_array_index(hub_op_list,i);
			if(p)
				free(p);
		}
		g_ptr_array_free(hub_op_list,TRUE);
	}
	hub_op_list=g_ptr_array_new();
	/* reset hub user xtra info */
	if(hub_user_xtra_info!=NULL)
	{
#if 0
		printf("resetting hub_user_xtra_info\n");
#endif
		for(i=0;i<hub_user_xtra_info->len;i++)
		{
			free_xtra_info(g_ptr_array_index(hub_user_xtra_info,i));
		}
		g_ptr_array_free(hub_user_xtra_info,TRUE);
	}
	hub_user_xtra_info=g_ptr_array_new();
#if 0
	printf("resetting finished\n");
#endif
	G_UNLOCK(hub_user_list);
}

/*******************************************/
/* add the given nick to the hub user list */
/*******************************************/
void add_user_to_user_list(char *nick)  
{
	int i;
	char *p;

	if(hub_user_list==NULL)
	{
		reset_hub_user_list();          /* if the list is not yet available, reset it */
#if 0
		printf("hub_user_list resetted\n");
#endif
	}
	/* don't lock before because reset_* also locks */
	G_LOCK(hub_user_list);

	/* search the given nick in the user list */
	for(i=0;i<hub_user_list->len;i++)
	{
		if(!strcmp(nick,g_ptr_array_index(hub_user_list,i)))
		{
			G_UNLOCK(hub_user_list);
#if 0
			printf("already in user list...\n");
#endif
			return;                 /* we have found it */
		}
	}

	/* not found, add it */
	p=strdup(nick);
	if(p!=NULL)
	{
		g_ptr_array_add(hub_user_list,p);
		g_ptr_array_add(hub_user_uinfo,NULL);                  /* add the UINFO pointer at the same time */
		g_ptr_array_add(hub_user_xtra_info,NULL);               /* add the xtra_info pointer at the same time */
	}
	G_UNLOCK(hub_user_list);
}

/*****************************************/
/* add the given nick to the hub op list */
/*****************************************/
void add_user_to_op_list(char *nick)
{
	int i;
	char *p;

	if(hub_op_list==NULL)
		reset_hub_user_list();          /* if the list is not yet available, reset it */

	G_LOCK(hub_user_list);
	/* search the given nick in the user list */
	for(i=0;i<hub_op_list->len;i++)
	{
		if(!strcmp(nick,g_ptr_array_index(hub_op_list,i)))
		{
			G_UNLOCK(hub_user_list);
			return;                 /* we have found it */
		}
	}

	/* not found, add it */
	p=strdup(nick);
	if(p!=NULL)
		g_ptr_array_add(hub_op_list,p);
	G_UNLOCK(hub_user_list);
}

/**********************************************/
/* remove the given nick to the hub user list */
/**********************************************/
void remove_user_to_user_list(char *nick)
{
	int i;

	G_LOCK(hub_user_list);
	
	if(hub_user_list==NULL)
	{
		G_UNLOCK(hub_user_list);
		return;
	}

	/* search the given nick in the user list */
	for(i=0;i<hub_user_list->len;i++)       
	{
		char *p;

		p=g_ptr_array_index(hub_user_list,i);
		if(!strcmp(nick,p))
		{
			free(p);
			g_ptr_array_remove_index_fast(hub_user_list,i);
			p=g_ptr_array_remove_index_fast(hub_user_uinfo,i);
			if(p!=NULL)
				free(p);

			free_xtra_info(g_ptr_array_remove_index_fast(hub_user_xtra_info,i));            /* free xtra information at the same time */
			break;
		}
	}

	/* the user is perhaps also an op */
	for(i=0;i<hub_op_list->len;i++) 
	{
		char *p;

		p=g_ptr_array_index(hub_op_list,i);
		if(!strcmp(nick,p))
		{
			free(p);
			g_ptr_array_remove_index_fast(hub_op_list,i);
			break;
		}
	}
	G_UNLOCK(hub_user_list);
}

static void add_pattern_to_search(unsigned short receive_port, char *pattern, unsigned int on_start, unsigned int on_incoming, unsigned int global_periodic)
{
	AUTOSEARCH nw;
	char myaddr[8192];

	sprintf(myaddr,"%s:%hu",my_hostip,receive_port);

	nw.last_global_scan=0;
	nw.search_result_return_address=g_strdup(myaddr);
	nw.search_query=g_string_new(pattern);
	nw.on_start=on_start;
	nw.on_incoming=on_incoming;
	nw.global_periodic=global_periodic;

	G_LOCK(search_to_start);
	search_to_start=g_array_append_val(search_to_start,nw);
	G_UNLOCK(search_to_start);
}

/******************************/
/* set user extra information */
/******************************/
void set_xtra_information_cnx( char *nick, int type, char *addr, const char *client, char *version, char *lock, char *pk, int has_bzlist, int got_lock, int got_myinfo, char *d_client, char *d_version)
{
	int i;
	
	G_LOCK(hub_user_list);
	/* first we must find the index of the given user */
	for(i=0;i<hub_user_list->len;i++)
	{
		char *p;

		p=g_ptr_array_index(hub_user_list,i);
#if 0
		printf("xtra-info comparing: %s %s\n",nick,p);
#endif
		if(!strcmp(nick,p))
		{
			/* gotcha */
			XTRA_INFO *xi;
#if 0
			printf("xtra-info found: %s\n",p);
#endif
			xi=g_ptr_array_index(hub_user_xtra_info,i);
			if(xi==NULL)
			{
				/* not yet set */
#if 0
				printf("no info yet - getting memory for entry\n");
#endif
				xi=malloc(sizeof(XTRA_INFO));
				if(xi!=NULL)
				{
					xi->type=0;
					xi->addr=NULL;
					xi->client=g_string_new("");
					if(client!=NULL)
						g_string_sprintf(xi->client,client);
					xi->version=g_string_new("");
					if(version!=NULL)
						g_string_sprintf(xi->version,version);
					xi->lock=g_string_new("");
					if(lock!=NULL)
						g_string_sprintf(xi->lock,lock);
					xi->pk=g_string_new("");
					if(pk!=NULL)
						g_string_sprintf(xi->pk,pk);
					xi->has_bzlist=has_bzlist;
					xi->got_lock=got_lock;
					xi->got_myinfo=got_myinfo;
					xi->d_client=g_string_new("");
					if(d_client!=NULL)
						g_string_sprintf(xi->d_client,d_client);
					xi->d_version=g_string_new("");
					if(d_version!=NULL)
						g_string_sprintf(xi->d_version,d_version);
					g_ptr_array_index(hub_user_xtra_info,i)=xi;
#if 0
					printf("finished creating entry\n");
#endif
				}
			}
			else
			{
				/* still set, do update */
#if 0
				printf("data found, trying to update\n");
#endif
				xi->type=type;
				if(addr!=NULL)
				{
					if(xi->addr!=NULL)
					{
						if(strcmp(addr,xi->addr))
						{
							free(xi->addr);
							xi->addr=strdup(addr);
						}
					}
					else
						xi->addr=strdup(addr);
				}
				else
				{
					if(xi->addr!=NULL)
					{
						free(xi->addr);
						xi->addr=NULL;
					}
				}
				if(client!=NULL)
				{
					if(xi->client->str!=NULL)
					{
						if(strcmp(client,xi->client->str))
						{
							g_string_free(xi->client,TRUE);
							xi->client=g_string_new(client);
						}
					}
					else
						xi->client=g_string_assign(xi->client,client);
				}
				if(version!=NULL)
				{
					if(xi->version->str!=NULL)
					{
						if(strcmp(version,xi->version->str))
						{
							g_string_free(xi->version,TRUE);
							xi->version=g_string_new(version);
						}
					}
					else
						xi->version=g_string_assign(xi->version,version);
				}
				if(lock!=NULL)
				{
					if(xi->lock->str!=NULL)
					{
						if(strcmp(lock,xi->lock->str))
						{
							g_string_free(xi->lock,TRUE);
							xi->lock=g_string_new(lock);
						}
					}
					else
						xi->lock=g_string_assign(xi->lock,lock);
				}
				if(pk!=NULL)
				{
					if(xi->pk->str!=NULL)
					{
						if(strcmp(pk,xi->pk->str))
						{
							g_string_free(xi->pk,TRUE);
							xi->pk=g_string_new(pk);
						}
					}
					else
						xi->pk=g_string_assign(xi->pk,pk);
				}

				if(got_lock!=0)
				{
						xi->got_lock=got_lock;
				}
				if(has_bzlist!=0)
				{
						xi->has_bzlist=has_bzlist;
				}
				if(got_myinfo!=0)
				{
					xi->got_myinfo=got_myinfo;
				}
				if(d_client!=NULL)
				{ 
					if(xi->d_client->str!=NULL)
					{
						if(strcmp(d_client,xi->d_client->str))
						{
							g_string_free(xi->d_client,TRUE);
							xi->d_client=g_string_new(d_client);
						}
					}
					else
						xi->d_client=g_string_assign(xi->d_client,d_client);
				}
				if(d_version!=NULL)
				{
					if(xi->d_version->str!=NULL)
					{
						if(strcmp(d_version,xi->d_version->str))
						{
							g_string_free(xi->d_version,TRUE);
							xi->d_version=g_string_new(d_version);
						}
					}
					else
						xi->d_version=g_string_assign(xi->d_version,d_version);
				}
				g_ptr_array_index(hub_user_xtra_info,i)=xi;
				/* so lets check if myinfo & lock has been received, and decide to kick evtl. faking or hiding clients... */
#if 0
				printf("got_lock: %i\n",xi->got_lock);
				printf("got_myinfo: %i\n",xi->got_myinfo);
#endif
				if((xi->got_lock==1)&&(xi->got_myinfo==1))
				{
					GString *warnmsg=NULL;
					int todo=FALSE;
					int ban=FALSE;

					warnmsg=g_string_new("");

					if((strcmp(xi->client->str,xi->d_client->str)==0) && (strcmp(xi->version->str,xi->d_version->str)==0))
					{
#if 0
						printf("clients and versions matching\n");
#endif
					}
					else if ((strcmp(xi->client->str,"DCTC")==0) && (strcmp(xi->d_client->str,"NM-DC")==0))
					{
#if 0
						printf("found dctc client\n");
#endif
					}
                                        else if ((strcmp(xi->client->str,"DC++")==0) && (strcmp(xi->d_client->str,"NM-DC")==0) && (strcmp(xi->
version->str,"modified pk")==0))
					{
						g_string_sprintf(warnmsg,"ban %s because %s hiding",
												nick,
												xi->pk->str);
						todo=TRUE;
						ban=TRUE;
					}
					else if ((strcmp(xi->client->str,"DC++")==0) && (strcmp(xi->d_client->str,"NM-DC")==0) && ((strcmp(xi->version->str,"0.18")==0) || (strcmp(xi->version->str,"0.181")==0) || (strcmp(xi->version->str,"0.177")==0)))
					{
						g_string_sprintf(warnmsg,"ban %s because %s hiding",
												nick,
												xi->client->str);
						todo=TRUE;
						ban=TRUE;
					}
					else if ((strcmp(xi->client->str,"DC++ Blue")==0) || (strcmp(xi->client->str,"DC++k hiding")==0) || (strcmp(xi->client->str,"mldonkey")==0))
					{
						g_string_sprintf(warnmsg,"ban %s because %s",
												nick,
												xi->client->str);
						todo=TRUE;
						ban=TRUE;
					}
					else if ((strcmp(xi->client->str,"DC++")==0) && (strcmp(xi->d_client->str,"DC++")==0) && (strcmp(xi->version->str,xi->d_version->str)!=0))
					{
						if(strstr(xi->d_version->str,"DC@fe++")==NULL)
						{
							g_string_sprintf(warnmsg,"Check %s for modified dc++ Version: %s, %s",
													nick,
													xi->version->str,
													xi->d_version->str);
							todo=TRUE;
						}
					}
					else
					{
						g_string_sprintf(warnmsg,"Information for %s complete:\r\n",nick);
						g_string_sprintfa(warnmsg,"Connect client: %s	version: %s\r\n",xi->client->str,xi->version->str);
						g_string_sprintfa(warnmsg,"Descrpt client: %s	version: %s\r\n",xi->d_client->str,xi->d_version->str);
						g_string_sprintfa(warnmsg,"bzlist: %i\r\n",xi->has_bzlist);
						g_string_sprintfa(warnmsg,"lock: %s\r\n",xi->lock->str);
						g_string_sprintfa(warnmsg,"pk: %s\r\n",xi->pk->str);
						G_LOCK(hub_com);
						send_dc_line(hub_com,warnmsg->str,NULL);
						G_UNLOCK(hub_com);
						g_string_free(warnmsg,TRUE);
					}
                                        if (todo==TRUE)
					{
						char *op_name;
						for(i=0;i<hub_op_list->len;i++)
						{
							op_name=g_ptr_array_index(hub_op_list,i);
                                                        if ((strcmp(my_name,op_name)) && (strcmp("Hub-Security",op_name)))
							{
								G_LOCK(hub_com);
								send_private_chat_message(hub_com, my_name, op_name, warnmsg->str, 1);
								G_UNLOCK(hub_com);
							}
						}
						if (ban==TRUE)
						{
							G_LOCK(hub_com);
							ban_user(hub_com,my_name,nick,"FAKER");
							G_UNLOCK(hub_com);
						}
					}
					g_string_free(warnmsg,TRUE);
				}
			}
			break;
		}
	}
        G_UNLOCK(hub_user_list);
/* #if 0
        dump_users_info();
#endif */
}

/************************************************************************/
/* check if the given pattern exists inside filename (case insensitive) */
/************************************************************************/
/* output: address of the pattern or NULL */
/******************************************/
const char *my_strcasestr(const GString *pattern, const char *filename)
{
   int max_pos;
   int i;

   max_pos=strlen(filename)-pattern->len;

   if(max_pos<0)             /* filename length < pattern length ? */
      return NULL;            /* no match */

   for(i=0;i<=max_pos;i++)
   {
      if(!strncasecmp(pattern->str,filename+i,pattern->len))
		return filename+i;
   }

   return NULL;               /* no match */
}
/**********************************************************/
/* backwardscheck for dc++ users - against doubling-Fakes */
/**********************************************************/
const char *my_strbackstr(const GString *pattern, const char *filename)
{
   int max_pos;
   int i;

   max_pos=strlen(filename)-pattern->len;

   if(max_pos<0)
      return NULL;

   i=max_pos;
   while(i>0)
   {
	i=i-1;
	if(!strncasecmp(pattern->str,filename+i,pattern->len))
		return filename+i;
   }

   return NULL;
}
/* ----------------------------------------------------------------------------------------- */
/* ----------------------------------------------------------------------------------------- */
/* ----------------------------------------------------------------------------------------- */
/* ---- find users having no enough slots opened ------------------------------------------- */
/* ----------------------------------------------------------------------------------------- */
/* ----------------------------------------------------------------------------------------- */
/* ----------------------------------------------------------------------------------------- */

/***************************************************************************************************/
/* this function analyzes the result of the "F?F?0?1?mp" search query to identify buggy clones */
/***************************************************************************************************/
static int slot_autosearch_sr(void *xtra_param,const char *nick, const char *filename, unsigned long long filesize, unsigned int free_slot, unsigned int ttl_slot)
{
	const GString explorer={"explorer.exe",strlen("explorer.exe")};
	const GString about={"AboutDC.exe",strlen("AboutDC.exe")};
	GString *resexplorer;
	GString *resabout;
	int i;
	int is_op=FALSE;
#if 0
	 printf("%s slot_autosearch_sr: '%s'\n",my_name,filename);
#endif
	resexplorer=g_string_new(my_strcasestr(&explorer,filename));
	resabout=g_string_new(my_strcasestr(&about,filename));

	G_LOCK(hub_user_list);
	for(i=0;i<hub_op_list->len;i++)
	{
		if(!strcmp(nick,g_ptr_array_index(hub_op_list,i)))
		{
			G_UNLOCK(hub_user_list);
			is_op=TRUE;                 /* we have found it */
			break;
		}
	}
	G_UNLOCK(hub_user_list);

	if	(
		(ttl_slot<2) /* 2 Slot is the Limit */
		&& (is_op==FALSE)
		)
	{
		G_LOCK(hub_com);
		kick_user(hub_com,my_name,nick,"This HUB has a Slot-Limit of 2 - change it and you can come back...");
		G_UNLOCK(hub_com);
	}
	
	if	(
		((resexplorer->str=="explorer.exe")||
		(resabout->str=="AboutDC.exe"))&& (is_op==FALSE)
		)
	{
		G_LOCK(hub_com);
		kick_user(hub_com,my_name,nick,"You are Sharing Windows-System-Files or installed Program-Files - change it and you can come back...");
		G_UNLOCK(hub_com);
	}
	g_string_free(resexplorer,TRUE);
	g_string_free(resabout,TRUE);

	return 0;
}

/**********************************************************************************************************************/
/* slot autosearch searchs for buggy DC clone returning everything but what it is expected when a research is started */
/**********************************************************************************************************************/
/* This function receives the search results of the query "$Search myhost mpyip F?F?0?1?mp|" sent to each   */
/* incoming user.                                                                                           */
/* theoritically, this should returns the list of max 5 files including the String "*mp*". I think nearly   */
/* everyone have an mp3 or an mpeg-file ;-). From this answer we extract the number of Slots, the user has. */
/* If the min-slot is not offered the user is kicked and banned and its IP is banned.                       */
/************************************************************************************************************/
/* NOTE: to reduce bandwidth usage, when added, a search is immediatly started and a "one user" search is   */
/*       performed each time a user enters the hub                                                          */
/************************************************************************************************************/
static void *slot_autosearch_fnc(void *dummy)
{
	int incoming_result_fd;
	static TBLDECODE gen_dec={NULL,NULL,NULL,NULL,slot_autosearch_sr,NULL,NULL,NULL,NULL,NULL,NULL};

	incoming_result_fd=_x_udp(0);			/* create an UDP socket and found an unused port */
	if(incoming_result_fd==-1)
	{
		fprintf(stderr,"slot_autosearch_fnc: fail to create UDP socket.\n");
	}
	else
	{
		struct sockaddr_in lcl;
		int len_lcl=sizeof(lcl);

		if(getsockname(incoming_result_fd,(void*)&lcl,&len_lcl)!=0)
		{
			perror("getsockname fail");
			shutdown(incoming_result_fd,2);
			close(incoming_result_fd);
		}
		else
		{
			GString *str;

			add_pattern_to_search(ntohs(lcl.sin_port),"F?F?0?1?mp",TRUE,TRUE,FALSE);
			add_pattern_to_search(ntohs(lcl.sin_port),"F?F?0?1?explorer.exe",TRUE,TRUE,FALSE);
			add_pattern_to_search(ntohs(lcl.sin_port),"F?F?0?1?AboutDC.exe",TRUE,TRUE,FALSE);
			str=g_string_new("");

			/* loop for ever and process received result */
			while(1)
			{
				int ret;
				char buf[8192];
#if HAVE_NO_MSG_NOSIGNAL
				void (*old_handler)(int);
				old_handler = signal (SIGPIPE, SIG_IGN);
#endif
				ret=recv(incoming_result_fd,buf,sizeof(buf)-1,MSG_NOSIGNAL);
				
#if HAVE_NO_MSG_NOSIGNAL
				signal (SIGPIPE, old_handler);
#endif

				if(ret!=-1)
				{
					buf[ret]='\0';
					
					/* to know if the result is buggy, we have to check if the size is above 1 */
					/* or if the returned filename does not contain the string "mp" */

					str=g_string_assign(str,buf);
					str=g_string_append(str,"|");
#if 0
					printf("%s Slot AutoSearch funktion got: %s\n",my_name,str->str);
#endif
					generic_decoder(str,&gen_dec,NULL);
				}
			}
		}
	}
	pthread_exit(NULL);
}

static void start_auto_search()
{
	pthread_t tid;
	pthread_attr_t thread_attr;
	pthread_attr_init (&thread_attr);
	pthread_attr_setdetachstate(&thread_attr, PTHREAD_CREATE_DETACHED);
	if(pthread_create(&tid,&thread_attr,(void*)slot_autosearch_fnc,NULL)!=0)
	{
		fprintf(stderr,"Fail to create auto search thread.\n");
		exit(1);
	}
}

static void *heartbeat_fnc(void *dummy)
{
	time_t last_heartbeat;
	gl_cur_time = time(NULL);
	last_heartbeat = time(NULL);

	while(1)
	{
		sleep(30);
		gl_cur_time = time(NULL);
		if((gl_cur_time - last_heartbeat) > 30)
		{
			last_heartbeat = gl_cur_time;
			G_LOCK(hub_com);
			send_dc_line(hub_com,"|",NULL);
			G_UNLOCK(hub_com);
		}
	}
	pthread_exit(NULL);
}

static void start_heartbeat()
{
	pthread_t tid;
	pthread_attr_t thread_attr;
	pthread_attr_init (&thread_attr);
	pthread_attr_setdetachstate(&thread_attr, PTHREAD_CREATE_DETACHED);
	if(pthread_create(&tid,&thread_attr,(void*)heartbeat_fnc,NULL)!=0)
	{
		fprintf(stderr,"Fail to create heartbeat thread.\n");
		exit(1);
	}
}

/********************************************************/
/* when someone connects on com port, send our nickname */
/********************************************************/
static int send_nick(int sck)
{
	GString *str;
	int a;

	str=g_string_new(NULL);
	g_string_sprintf(str,"$MyNick %s|",my_name);
#if 0
	printf("send nick sent: '%s'\n",str->str);
#endif

#if 0
	a=write(sck,str->str,str->len);                 /* @@@ replaced by send */
	a=(a!=str->len);
#else
{
#if HAVE_NO_MSG_NOSIGNAL
	void (*old_handler)(int);
	old_handler = signal (SIGPIPE, SIG_IGN);
#endif

	a=((send(sck,str->str,str->len,MSG_NOSIGNAL)) != str->len);
	
#if HAVE_NO_MSG_NOSIGNAL
	signal (SIGPIPE, old_handler);
#endif
}	
#endif

	g_string_free(str,TRUE);
	return a;                               /* 1=fail, 0=ok */
}

/***********************************************/
/* destruction of the given WAIT_ACT structure */
/***********************************************/
void free_wait_act(WAIT_ACT *act)
{
	if(act==NULL)
		return;
	if(act->sock_fd)
	{
		shutdown(act->sock_fd,2);
		close(act->sock_fd);
		act->sock_fd=0;
	}
	if(act->remote_nick!=NULL)
	{
		g_string_free(act->remote_nick,TRUE);
		act->remote_nick=NULL;
	}
	if(act->remote_addr!=NULL)
	{
		g_string_free(act->remote_addr,TRUE);
		act->remote_addr=NULL;
	}
	if(act->run_task!=NULL)
	{
		fprintf(stderr,"WARNING, in free_wait_act, run_task still is here.\n");
		act->run_task=NULL;
	}
	if(act->disp_info!=NULL)
	{
		g_string_free(act->disp_info,TRUE);
		act->disp_info=NULL;
	}
	if(act->cap_str!=NULL)
	{
		g_string_chunk_free(act->cap_str);
		act->cap_str=NULL;
	}
	if(act->cap_ptr!=NULL)
	{
		g_ptr_array_free(act->cap_ptr,TRUE);
		act->cap_ptr=NULL;
	}
	free(act);
}

/***************************************************/
/* decode the received Lock to get the client Type */
/***************************************************/
void decode_lock(char *nick, char *received_lock, int has_bzlist)
{
	int i;
	int dec_lock=-1;
	int dec_pk=-1;
	char *client=NULL;
	char *version=NULL;
	char *lock=NULL;
	char *pk=NULL;
	char *dp=NULL;

	/* trying to split the String into lock & pk part */
	dp=received_lock;
	lock=dp;

	dp=strstr(lock," Pk=");	
	if(dp==NULL)
		goto no_pk;
	*dp++='\0';
	dp+=3;
	pk=dp;

	no_pk:
#if 0
	printf("extracted lock: '%s'\n",lock);
	printf("extracted pk  : '%s'\n",pk);
#endif
	i=0;
	while(lockstart[i]!=NULL)
	{
		if(!strncmp(lock,lockstart[i],strlen(lockstart[i])))
		{
			dec_lock=i;
			if(dec_lock==0)
				client="DC++";
			if(dec_lock==1)
				client="DC++k hiding";
			break;
		}
		i++;
	}
#if 0
	printf("client lock: %i\n",dec_lock);
#endif
	if(pk!=NULL)
	{
#if 0
		printf("pk is not NULL\n");
#endif
		i=0;
		while(pkstart[i]!=NULL)
		{
#if 0
			printf("pkstart: '%s'\n",pkstart[i]);
#endif
			if(!strncmp(pk,pkstart[i],strlen(pkstart[i])))
			{
				dec_pk=i;
				if(dec_pk==0)
				{
					char *z;
					char *t;
#if 0
					printf("decoded pk: %i\n",dec_pk);
#endif
					z=strdup(pk);
					t=z;
					t+=strlen(pkstart[i]);
					version=t;
					if(strstr(version,"ABCABC")!=NULL)
					{
						char *pkrest;
#if 0
						printf("pk includes ABCABC\n");
#endif
						t=strstr(version,"ABCABC");
						*t++='\0';
						pkrest=t;
						if(strlen(pkrest)!=5)
						{
#if 0
							printf("lnge pkrest: %i %s\n",strlen(pkrest),pkrest);
#endif
							version="modified pk";
						}
					}
				}
				else if(dec_pk==1)
				{
					version="hidden";
				}
				else if(dec_pk==2)
				{
					client="mldonkey";
				}
				break;
			}
			i++;
		}
		if(dec_pk==-1 && dec_lock==-1)
		{
			if (strlen(pk)==16)
				client="NM-DC";
			if ((strlen(pk)>50) && (strstr(pk,"DCN096")) && (strstr(pk,"DCN036")) && (strstr(pk,"DCN000")))
				client="DC++ Blue";
#if 0
			printf("lnge lock: %i\n",strlen(lock));
			printf("lnge pk: %i\n",strlen(pk));
#endif
		}
	}
	else if(pk==NULL && strlen(lock)>50)
	{
		client="DCTC";
	}
#if 0
	printf("client pk: %i\n",dec_pk);
	printf("found client: %s - %s\n",client,version);
	printf("decode lock is setting xtra information\n");
#endif
	set_xtra_information_cnx(nick,0,NULL,client,version,lock,pk,has_bzlist,1,0,NULL,NULL);
}

/***************************************************************************/
/* create a lock inside the 'created_lock' array and send it on the socket */
/***************************************************************************/
static int send_a_lock(int sck,char *created_lock)
{
	gchar *v;
	int ret;
	created_lock="EXTENDEDPROTOCOLABCABCABCABCABCABC Pk=DCPLUSPLUS0.181ABCABC|";
	v=g_strconcat("$Lock ",created_lock,NULL);

{
#if HAVE_NO_MSG_NOSIGNAL
				void (*old_handler)(int);
				old_handler = signal (SIGPIPE, SIG_IGN);
#endif
	ret=(send(sck,v,strlen(v),MSG_NOSIGNAL)!=strlen(v));
	
#if HAVE_NO_MSG_NOSIGNAL
				signal (SIGPIPE, old_handler);
#endif
}
	g_free(v);
	if(ret)
		return 1;       /* fails */

	return 0;               /* ok */
}

/******************************/
/* thread managing connect_me */
/******************************/
static void *connect_me_thread(WAIT_ACT *act)
{
	GString *input=NULL;
	char c_lock[256];
	GString *rec_lock=NULL;
	int has_bzlist=0;
#if 0
	printf("connect me thread started: '%lu'\n",(unsigned long)act);
#endif

	while(act->thread_id==0)
		usleep(1000);           /* wait until the main process has set the thread_id in act */
#if 0
	printf("connect_me_thread: '%lu' continued\n",(unsigned long)act);
#endif
	act->running=1;
	act->last_touch=time(NULL);

	/* add act to the waiting task array */
	G_LOCK(waiting_action);
	g_ptr_array_add(waiting_action,act);
	G_UNLOCK(waiting_action);
#if 0
	printf("connect_me_thread: '%lu' act added\n",(unsigned long)act->thread_id);
#endif
	/* send our name */
	if(send_nick(act->sock_fd))
	{
#if 0
		printf("connect_me_thread: %lu fail to send nick\n",(unsigned long)act->thread_id);
#endif
		goto thread_over;
	}
#if 0
	printf("connect_me_thread: %lu nick sent\n",(unsigned long)act->thread_id);
#endif
	if(send_a_lock(act->sock_fd,c_lock))
	{       /* send a lock to the remote side */
#if 0
		printf("connect_me_thread: %lu fail to send lock\n",(unsigned long)act->thread_id);
#endif
		goto thread_over;
	}
#if 0
	printf("connect_me_thread: %lu lock sent\n",(unsigned long)act->thread_id);
#endif
	input=get_a_dc_line(act->sock_fd);                              /* get the "$MyNick line */
	if(input==NULL)
	{
#if 0
		printf("connect_me_thread: %lu no incoming data\n",(unsigned long)act->thread_id);
#endif
		goto thread_over;
	}
#if 0
	printf("connect_me_thread got: %s\n",input->str);
#endif
	/* we should have received "$MyNick xxxx|" */
	if(strncmp("$MyNick ",input->str,8))
	{
#if 0
		printf("connect_me_thread: %lu $Nick not found\n",(unsigned long)act->thread_id);
#endif
		if(input!=NULL)
		{
			g_string_free(input,TRUE);
			input=NULL;
		}
		goto thread_over;
	}

	/* My nick is here */
	input=g_string_erase(input,0,8);                	/* remove the "$MyNick " at the beginning */
	input=g_string_truncate(input,input->len-1);            /* remove the trailing | */

	G_LOCK(waiting_action);
	act->remote_nick=input;                                 /* set the remote nick */
	G_UNLOCK(waiting_action);
	
	input=get_a_dc_line(act->sock_fd);              	/* get the $Lock line */
	if(input==NULL)
	{
#if 0
		printf("connect_me_thread: %lu $Lock not found\n",(unsigned long)act->thread_id);
#endif
		goto thread_over;
	}
#if 0
	printf("connect_me_thread: %lu %s\n",(unsigned long)act->thread_id,input->str);
#endif
	input=g_string_erase(input,0,6);			/* remove the "$Lock " at the beginning */
	input=g_string_truncate(input,input->len-1);            /* remove the trailing | */
	rec_lock=input;

	/* query if bzlist is supported */
	input=get_a_dc_line(act->sock_fd);              	/* get the $Supports result*/
	if(input==NULL)
	{
#if 0
		printf("connect_me_thread: %lu $Supports not found\n",(unsigned long)act->thread_id);
#endif
		/* goto thread_over; */
	}
	else
	{
#if 0
		printf("connect_me_thread: %lu %s\n",(unsigned long)act->thread_id,input->str);
#endif
		if(!strncmp(input->str,"$Supports BZList",16))
			has_bzlist=1;
		g_string_free(input,TRUE);
		input=NULL;
	}
#if 0
	printf("try to decode lock\n");
#endif
	decode_lock(act->remote_nick->str,rec_lock->str,has_bzlist);
	/* thread is over */
	thread_over:
#if 0
	printf("thread over\n");
#endif
	G_LOCK(waiting_action);
	g_ptr_array_remove_fast(waiting_action,act);
	G_UNLOCK(waiting_action);
	free_wait_act(act);
#if 0
	printf("connect_me_thread: '%s' finished\n",c_lock);
#endif
	pthread_exit(NULL);
}

/********************************************/
/* someone tries to connect to the com port */
/********************************************/
void manage_com_port(void *dummy)
{
	int sock_fd;
	WAIT_ACT *nw_act;
	pthread_attr_t thread_attr;
	struct sockaddr_in remote;
	int x=sizeof(remote);

	sock_fd=accept(in_sck,(struct sockaddr *)&remote,&x);
	if(sock_fd==-1)
	{
#if 0
		printf("manage_com_port - someone fails to connect to com port\n");
#endif
		return;
	}
	/* ok, remote host connect to the newly created socket */
	nw_act=malloc(sizeof(WAIT_ACT));
	if(nw_act==NULL)
	{
#if 0
		printf("manage_com_port - malloc failed\n");
#endif
		shutdown(sock_fd,2);
		close(sock_fd);
		return;
	}

#if 0
	printf("manage_com_port - creating thread info %lu\n",(unsigned long)nw_act);
#endif
	nw_act->running=3;                      /* thread init */
	nw_act->cnx_type=CNX_ON_ACCEPT;
	nw_act->last_touch=time(NULL);
	nw_act->thread_id=0;
	nw_act->sock_fd=sock_fd;
	nw_act->remote_nick=NULL;

	/* remote address */
	nw_act->remote_addr=g_string_new("");
	g_string_sprintfa(nw_act->remote_addr,"%s:%hu", inet_ntoa(remote.sin_addr),ntohs(remote.sin_port));

	nw_act->run_task=NULL;
	nw_act->disp_info=NULL;

	/* remote client capabilities */
	nw_act->cap_str=NULL;
	nw_act->cap_ptr=NULL;

	/* create the new thread */
	pthread_attr_init (&thread_attr);
	pthread_attr_setdetachstate(&thread_attr, PTHREAD_CREATE_DETACHED);
	if(pthread_create(&(nw_act->thread_id),&thread_attr, (void*)connect_me_thread,nw_act)!=0)
	{
		if(errno==EINTR)
		{
#if 0
			printf("manage_com_port - mcp:pthread_create failed - MEMORY NOT FREED TO AVOID CRASH\n");
#endif
		}
		else
		{
#if 0
			printf("manage_com_port - mcp:pthread_create failed\n");
#endif
			free(nw_act);
			shutdown(sock_fd,2);
			close(sock_fd);
			return;
		}
	}

	/* nw_act will be freed by the thread */
}

static void *in_com_fnc(void *dummy)
{
	int ln;
	int n;
	fd_set rd,wd;
	struct timeval tv;
	
	in_sck=_x_tcp(0);
	
	if(in_sck==-1)
	{
		fprintf(stderr,"slot_autosearch_fnc: fail to create TCP socket.\n");
	}
	else
	{
		struct sockaddr_in my_tcp;
		int len_my_tcp=sizeof(my_tcp);

		if(getsockname(in_sck,(void*)&my_tcp,&len_my_tcp)!=0)
		{
			perror("getsockname fail");
			shutdown(in_sck,2);
			close(in_sck);
		}
		else
		{
			sprintf(my_incom,"%s:%hu",my_hostip,ntohs(my_tcp.sin_port));
			listen(in_sck,64);
		}
        
		while(1)
		{
			n=-1;

			FD_ZERO(&rd);
			FD_ZERO(&wd);

			if(in_sck!=-1)                  /* com port exists ? */
			{
				FD_SET(in_sck,&rd);
				n=max(n,in_sck);
			}
			tv.tv_sec=1;
			tv.tv_usec=0;
			ln=select(n+1,&rd,&wd,NULL,&tv);
			if(ln>0)
			{
				if((in_sck!=-1)&&(FD_ISSET(in_sck,&rd)))
				{       /* someone wants to connect to this client ? */
					manage_com_port(NULL);
				}
			}
			else if(ln==-1)
			{
				if(errno!=EINTR)
					break;
			}
		}
	}
	pthread_exit(NULL);
}

static void start_in_com()
{
	pthread_t tid; 
	pthread_attr_t thread_attr;
	pthread_attr_init (&thread_attr);
	pthread_attr_setdetachstate(&thread_attr, PTHREAD_CREATE_DETACHED);
	if(pthread_create(&tid,&thread_attr,(void*)in_com_fnc,NULL)!=0)
	{
		fprintf(stderr,"Fail to create in_com thread.\n");
		exit(1);
	}
}


/***************************************************************/
/* this message is received each time a user changes his share */
/*****************************************************************************************/
/* when a user enters the hub, we send him all the query having the on_incoming flag set */
/*****************************************************************************************/
static int global_myinfo_fnc(void *xtra_param, const char *nickname, const char *description, const char *cnxtype, unsigned char flag, const char *email, double amount)
{
	int i;
	char *t;
	char *d_version=NULL;
	char *d_client=NULL;
	char *reason=NULL;
	char *nick;
	char *cmode;
	char *hubs;
	char *slots;
	char *line;
	char *dcplusrest;
	int hubnum;
	int slotnum;
	int linenum;
	int dcfake=FALSE;
	GString *output;
	GString *dcplusfnd;
	const GString dcplusplus={"<++ V:",strlen("<++ V:")};
	dcplusfnd=g_string_new(my_strbackstr(&dcplusplus,description));
	if(my_strbackstr(&dcplusplus,description)!=NULL)
	{
#if 0
		printf("%s global myinfo fnc got dc++: %s\n",my_name,dcplusfnd->str);
#endif
		/*****************************************************/
		/* dc++ info decode                                  */
		/*****************************************************/
		/* the dc++ string hast following structure          */
		/* <++ V:a.aaa,M:x,H:yyyy,S:zzzz>                    */
		/* or						     */
		/* <++ V:a.aaa,M:x,H:yyyy,S:zzzz,L:bbbb>             */
		/*****************************************************/
		/* a.aaa is the Version                              */
		/* x     is the Connect Mode (A - Active)            */
		/*                           (P - Passive)           */
		/* yyyy  is the number of Hubs connected to          */
		/* zzzz  is the number of slots offered              */
		/* bbbb  is a line-value			     */
		/*****************************************************/
		t=dcplusfnd->str+strlen("<++ V:");
		
		d_version=t;
		t=strchr(d_version,',');
		*t++='\0';
		if(strlen(d_version)==0)
		{
#if 0
			printf("descriptiondecoding version=NULL - fake\n");
#endif
			dcfake=TRUE;
			reason="version=NULL";
			goto dcplus_fake;
		}
#if 0
		printf("%s global myinfo fnc got dc++Version: %s\n",my_name,d_version);
#endif
		cmode=t;
		if(strncmp(cmode,"M:",2))
		{
#if 0
			printf("descriptiondecoding mode 'M:' not found - fake\n");
#endif
			dcfake=TRUE;
			reason="no mode found";
			goto dcplus_fake;
		}
		cmode=t+2;
		t=strchr(cmode,',');
		*t++='\0';
		/* cmode should be just one character 'A' for active or 'P' for passive */
		if((strlen(cmode)!=1) || (strcspn(cmode,"AP"))!=0)
		{
#if 0
			printf("descriptiondecoding mode got fake\n");
#endif
			dcfake=TRUE;
			reason="wrong mode found";
			goto dcplus_fake;
		}
#if 0
		printf("%s global myinfo fnc got dc++Connection: %s\n",my_name,cmode);
#endif
		hubs=t;
		if(strncmp(hubs,"H:",2))
		{
#if 0
			printf("descriptiondecoding hubs 'H:' not found - fake\n");
#endif
			dcfake=TRUE;
			reason="no hub found";
			goto dcplus_fake;
		}
		hubs=t+2;
		t=strchr(hubs,',');
		*t++='\0';
		if(strlen(hubs)==0 || strspn(hubs,"0123456789")!=strlen(hubs))
		{
#if 0
			printf("descriptiondecoding hubs got fake\n");
#endif
			dcfake=TRUE;
			reason="wrong hub number";
			goto dcplus_fake;
		}
		hubnum = atoi (hubs);
#if 0
                printf("%s global myinfo fnc got dc++ Hubs: %u\n",my_name,hubnum);
#endif
		slots=t;
		if(strncmp(slots,"S:",2))
		{
#if 0
			printf("descriptiondecoding slots 'S:' not found - fake\n");
#endif
			dcfake=TRUE;
			reason="no slots found";
			goto dcplus_fake;
		}
		slots=t+2;
		if((t=strchr(slots,','))!=NULL)
		{
			*t++='\0';
			line=t;
			if((strncmp(line,"L:",2)!=0) && (strncmp(line,"B:",2)!=0) && (strncmp(line,"U:",2)!=0))
			{
#if 0
				printf("descriptiondecoding line 'L:, B: or U:' not found - fake\n");
#endif
				dcfake=TRUE;
				reason="no line found";
				goto dcplus_fake;
			}
			line=t+2;
			t=strchr(line,'>');
			*t++='\0';
			if(strlen(line)==0 || strspn(line,"0123456789")!=strlen(line))
			{
#if 0
				printf("descriptiondecoding line got fake\n");
#endif
				dcfake=TRUE;
				reason="wrong line number";
				goto dcplus_fake;
			}
			linenum = atoi (line);
		}
		else
		{
			line=NULL;
			t=strchr(slots,'>');
			*t++='\0';
		}
		if(strlen(slots)==0 || strspn(slots,"0123456789")!=strlen(slots))
		{
#if 0
			printf("descriptiondecoding slot got fake\n");
#endif 
			dcfake=TRUE;
			reason="wrong slots number";
			goto dcplus_fake;
		}
		slotnum = atoi (slots);
#if 0
                printf("%s global myinfo fnc got dc++ Slots: %u\n",my_name,slotnum);
#endif
		dcplusrest=t;
		/* where should be nothing behind the dc++-description */
		if(strlen(dcplusrest)!=0)
		{
#if 0
			printf("descriptiondecoding rest got fake\n");
#endif
			dcfake=TRUE;
			reason="something behind dc++ tag";
			goto dcplus_fake;
		}
		dcplus_fake:
		if (dcfake==TRUE)
		{
			int i;
			int is_op=FALSE;
			G_LOCK(hub_user_list);
			for(i=0;i<hub_op_list->len;i++)
			{
				if(strcmp(nickname,g_ptr_array_index(hub_op_list,i))==0)
				{
					G_UNLOCK(hub_user_list);
					is_op=TRUE;                 /* we have found it */
					break;
				}
			}
			G_UNLOCK(hub_user_list);
			if(is_op==FALSE)
			{
				char *op_name;
				GString *out;
				out=g_string_new("");
				g_string_sprintf(out,"myinfo: Check %s for modified DC++ tag: %s ",nickname,reason);
#if 0
				printf("%s - No Faking DC++ clients...\n", nickname);
#endif
				G_LOCK(hub_user_list);
				for(i=0;i<hub_op_list->len;i++)
				{
					op_name=g_ptr_array_index(hub_op_list,i);
                                        if (strcmp(my_name,op_name) && strcmp("tantchen",op_name))
					{
						G_LOCK(hub_com);
						send_private_chat_message(hub_com, my_name, op_name, out->str, 1);
						G_UNLOCK(hub_com);
					}
				}
				G_UNLOCK(hub_user_list);
                                /* G_LOCK(hub_com);
				kick_user(hub_com,my_name,nickname,"No Faking DC++ clients...");
                                G_UNLOCK(hub_com); */
				goto userkicked;
			}
		}
		else if((hubnum>4 || slotnum>20 || slotnum<2))
		{
			int i;
			int is_op=FALSE;
			G_LOCK(hub_user_list);
			for(i=0;i<hub_op_list->len;i++)
			{
				if(!strcmp(nickname,g_ptr_array_index(hub_op_list,i)))
				{
					G_UNLOCK(hub_user_list);
					is_op=TRUE;                 /* we have found it */
					break;
				}
			}
			G_UNLOCK(hub_user_list);
			if(is_op==FALSE)
			{
				printf("%s - DC++ Rules-break\n", nickname);
				G_LOCK(hub_com);
				kick_user(hub_com,my_name,nickname,"DC++-User: 4 HUBS and 2 Slots is Limit. Don't open more than 20 SLOTS...");
				G_UNLOCK(hub_com);
				goto userkicked;
			}
		}
		d_client="DC++";
	}
	else
		d_client="NM-DC";
        if((in_sck!=-1) && (strcmp(nickname,my_name)) && (strcmp(nickname,"tantchen")) && (strcmp(nickname,"Hub-Security")))
	{
		GString *out;
		out=g_string_new("");
		g_string_sprintf(out,"$ConnectToMe %s %s|",nickname,my_incom);
		G_LOCK(hub_com);
		send_dc_line(hub_com,out->str,NULL);
		G_UNLOCK(hub_com);
#if 0
		printf("%s global myinfo handler sends: %s\n",my_name,out->str);
#endif
		g_string_free(out,TRUE);
	}

	output=g_string_new("");
	G_LOCK(search_to_start);
	for(i=0;i<search_to_start->len;i++)
	{
		AUTOSEARCH *as;

		as=&(g_array_index(search_to_start,AUTOSEARCH,i));
		g_string_sprintf(output,"$UniSearch %s %s %s|",nickname,as->search_result_return_address,as->search_query->str);
#if 0
		printf("%s global myinfo handler sends: '%s'\n",my_name,output->str);
#endif
		G_LOCK(hub_com);
		send_dc_line(hub_com,output->str,NULL);
		G_UNLOCK(hub_com);
	}
	
	G_UNLOCK(search_to_start);
#if 0
	printf("myinfo setting xtra information\n");
#endif
	nick=strdup(nickname);
	set_xtra_information_cnx(nick,0,NULL,NULL,NULL,NULL,NULL,0,0,1,d_client,d_version);
	g_string_free(output,TRUE);
	userkicked:
	g_string_free(dcplusfnd,TRUE);
	return 0;
}
/***************************************************************************************************************/
/* static nicklist funktion  - this Message should be received from hub after sending the $GetNickList command */
/***************************************************************************************************************/
static int global_nicklist_fnc(void *xtra_parm, const char *nicklist)
{
	char *u;
	char *t;
	GString *output;
#if 0
	printf("%s global nicklist handler got: %s\n",my_name,nicklist);
#endif
	t=strdup(nicklist);
	while((*t)&&(*t!='|'))
	{
		output=g_string_new("");
		u=strstr(t,"$$");
		if(u==NULL)
		{
#if 0
			printf("nicklist_process invalid nick list\n");
#endif
			return 0;
		}
	
		*u='\0';
		add_user_to_user_list(t);
		g_string_sprintf(output,"$GetINFO %s %s|",t,my_name);
		G_LOCK(hub_com);
		send_dc_line(hub_com,output->str,NULL);
		G_UNLOCK(hub_com);
		t=u+2;
		g_string_free(output,TRUE);
	}
	return 0;
}

/*************************************************************************************/
/* static oplist funktion  - this Message is received each time an Operator connects */
/*************************************************************************************/
static int global_oplist_fnc(void *xtra_parm, const char *oplist)
{
	char *u;
	char *t;
#if 0
	printf("%s global oplist handler got: %s\n",my_name,oplist);
#endif
	t=strdup(oplist);
	while((*t)&&(*t!='|'))
	{
		u=strstr(t,"$$");
		if(u==NULL)
		{
#if 0
			printf("nicklist_process invalid nick list\n");
#endif
			return 0;
		}

		*u='\0';
		add_user_to_op_list(t);
		t=u+2;
	}
	return 0;
}

/***********************************************************************************************/
/* static Private-Chat funktion  - this message is received each time an Message is send to me */
/***********************************************************************************************/
/* redirect op-kick-messages to the hub                                                        */
/***********************************************************************************************/
static int global_privmsg_fnc(void *xtra_parm, const char *source_nick, const char *dest_nick, const char *message)
{
	/* nothing to handle in here yet */
	return 0;
}

static int global_hello_fnc(void *xtra_param,const char *nickname)
{
	char *hnick;
	printf("%s global hello handler got: %s\n",my_name,nickname);
	hnick=strdup(nickname);
	add_user_to_user_list(hnick);
	return 0;
}

static int global_quit_fnc(void *xtra_param, const char *nickname)
{
	char *dp;
	dp=strdup(nickname);
	remove_user_to_user_list(dp);
	return 0;
}

/*********************************************************************************************************/
/* the program receives 3 parameters: its name (as it appears on the hub), the hub com socket number and */
/* the localhost IP (not 127.0.0.1, the IP used by users to connect to the hub).                         */
/* NOTE: the dc_com.c is the same as the src one but you must provide the DC_INPUT_NO_TIMEOUT else after */
/* 30 seconds idle, you will received a NULL string.                                                     */
/*********************************************************************************************************/
int main(int argc, char **argv)
{
	GString *str;

	if(argc!=4)
	{
		fprintf(stderr,"Usage: %s name fdnumber localhostIP\n",argv[0]);
		exit(1);
	}

	my_name=argv[1];
	sscanf(argv[2],"%u",&hub_com);

	my_hostip=g_strdup(argv[3]);
	g_thread_init(NULL);

	/* ask for nick & oplist ... - hub don't sent it on starting bot manually */
	reset_hub_user_list();
	send_dc_line(hub_com,"$GetNickList|",NULL);

	search_to_start=g_array_new(FALSE,FALSE,sizeof(AUTOSEARCH));
	start_auto_search();
	start_heartbeat();
	start_in_com();

	/* allocated running task array and queued task array */
	waiting_action=g_ptr_array_new();
	while(1)
	{
		static TBLDECODE global_gen_dec={global_myinfo_fnc,NULL,global_privmsg_fnc,NULL,NULL,NULL,NULL,global_hello_fnc,global_quit_fnc,global_nicklist_fnc,global_oplist_fnc};

		str=get_a_dc_line(hub_com);

		if(str==NULL)
			break;

#if 0
		printf("%s received: '%s'\n",my_name,str->str);
#endif
	
		generic_decoder(str,&global_gen_dec,NULL);

		g_string_free(str,TRUE);

	}
#if 0
	printf("%s has lost hub connection and leaves\n",my_name);
#endif
	return 0;
}

