# Used db keys: MIN_SHARE, HUB_MASTER, HUB_DIR, LANGUAGE. All four keys are strings.
# The keys HUB_MASTER and HUB_DIR has to be defined for script to work properly.
# HUB_MASTER shall contain the nick of the hub master, probably you. It can contain any string, but many messages
# will then be lost.
# HUB_DIR specifyes the hub directory, this is not the bin-dir, but the dir where you keep your scripts, extprogs...
# The key LANGUAGE defines which languge that will be used in messages, default is "eng", other option are "swe"
# MIN_SHARE contains the minimal shared size in bytes.
# To add key to the database, use "-dbadd KEYNAME str xxx" where KEYNAME is the keys name and xxx is the wabted value.
# the MIN_SHARE requirement is not used against hub OP or Master, only with standard user.

# Files used: ./scripts/.ipmask ./log/hub.log ./scripts/.rules ./scripts/.prefixchk ./scripts/.namebot

# IMPORTANT: create the file "HUB_DIR"/log/hub.log by executing "touch ..../log/hub.log" Otherwise this script will cause a hub crash.
# This is to be fixed in future scripts.

use strict;

sub dchub_myinfo_handler {
	our %evt_array;
	*evt_array = $_[0];
	print "myinfo handler\n";
	printf "event   = '".$evt_array{"event"}."'\n";
	printf "nickname= '".$evt_array{"nickname"}."'\n";
	printf "argc    = '".$evt_array{"argc"}."'\n";
	
	
	#Declaration of system variables
	my $hubdir=dchub::db_get("HUB_DIR");
	my $scriptdir=$hubdir."scripts/";
	my $logdir=$hubdir."log/";
	
	#Declaration of hub variables
	my $hubmaster=dchub::db_get("HUB_MASTER");
	my $language=dchub::db_get("LANGUAGE");
	
	#Check so that the most important vars are defined.
	return if (not defined($hubmaster));

	if (not defined($hubdir))
	{
		dchub::send_pchat_msg("Script-Panic",$hubmaster,"Hubdir not defined! Please set db var HUB_DIR to current hub directory.|");
		return;
		
	}
	
	#Declaration of txt variables. This will make it easy to translate. Default is eng.
	my ($welcome,$this_is_the_hub,$logged_on);
	my ($you_must_share_more_than,$bytes_Add_some_files_and_come_back_later,$hopefully_you_are_an_op);
	my ($you_have_got_to_have_a_specifik_ip,$disconnected_because_wrong_ip);
	my ($disconnected_because_wrong_prefix);
	if((not defined($language)) or ($language eq "eng"))
	{
		#Welcome strings.
		$welcome="Welcome ";
		$this_is_the_hub=". This is the greatest hub known to mankind."; #Replace with your own description.
		$logged_on=" logged on.";
		
		#Min share strings.
		$you_must_share_more_than="You must share more than ";
		$bytes_Add_some_files_and_come_back_later="bytes. Add some files and come back later.";
		$hopefully_you_are_an_op="... hopefully, you are an OPerator";
		
		#IP-check strings.
		$you_have_got_to_have_a_specifik_ip="Your IP must begin with 123.4"; #Replace with your own IP criteria.
		$disconnected_because_wrong_ip=" disconnected (Invalid IP)";
		
		#Nickcheck strings.
		$disconnected_because_wrong_prefix=" disconnected (Invalid prefix)";
		
		
	}
	elsif($language	eq "swe")
	{
		#Welcome strings.
		$welcome="Vlkommen ";
		$this_is_the_hub=". Detta r Inofficiella Bsksllskapets DCHUB p olofshojd.studenthem.gu.se"; #Replace with your own description.
		$logged_on=" loggade in.";
		
		#Min share strings.
		$you_must_share_more_than="Du mste dela ut mer n ";
		$bytes_Add_some_files_and_come_back_later="bytes. Lgg till lite filer och kom tillbaks.";
		$hopefully_you_are_an_op="... frhoppningsvis r du en OPeratr";
		
		#IP-check strings.
		$you_have_got_to_have_a_specifik_ip="Du mste vara ansluten till SUNet Gteborg fr att nyttja den hr hubben."; #Replace with your own IP criteria.
		$disconnected_because_wrong_ip=" bortkopplad (Ogiltigt IP)";
		
		#Nickcheck strings.
		$disconnected_because_wrong_prefix=" bortkopplad (Ogiltigt prefix)";
	}
	else
	{
		dchub::send_pchat_msg("Script-Panic",$hubmaster,"Invalid language! Set db var LANGUAGE to valid language or remove it.|");
		return;
		
	}
		
	#Declaration and initiation of user specific vars.
	my $duration=dchub::nick_duration($evt_array{"nickname"});
	my $ip=dchub::nick_ip($evt_array{"nickname"});
	my(@uinfo)=split(/\$/,dchub::nickinfo($evt_array{"nickname"}));
		
	#Declaration of misc variables
	my $str="";	#A temporary string variable.
	my $char=""; 	#A temporary string variable used only for single chars.
	
	#Eric, why is this "if" needed? /Daniel Marmader
	if($evt_array{"argc"}!=1)
	{
		dchub::disconnect($evt_array{"nickname"});
	}
	else
	{
		#Alert users when Master or OPerator logs on.
		if($uinfo[5]!=0)
		{
			if($duration<4)
			{
				dchub::send_pchat_msg("Hub-Log",$hubmaster,$evt_array{"nickname"}." IP:".$ip.$logged_on."|");
				dchub::send_to_all_users("<Hub-Security> ".$evt_array{"nickname"}.$logged_on."|");
			}
		}

		################################################
		#Welcome message.
		################################################
		if($duration<4)
		{
			#Welcome
			$str=$welcome;
			$str.=$evt_array{"nickname"};
			$str.=$this_is_the_hub;
			dchub::send_pchat_msg("Hub",$evt_array{"nickname"},"$str|");
			#Print rules.
			$str=$scriptdir.".rules";
			open(RULESFILE, $str) or $str="err";
			if ($str ne "err")
			{
				my $rules="";
				$char="";
				while(not eof(RULESFILE))
				{
					read(RULESFILE,$char,1);
					if(ord($char)!=10)
					{
						$rules=$rules.$char;
					}
					else
					{
						$rules=$rules.chr("13").chr("10");
					}
				}
				close(RULESFILE);
				dchub::send_pchat_msg("Hub",$evt_array{"nickname"},$rules."|");
			}
			dchub::send_to_named_user($evt_array{"nickname"},"<Penguin> Use Linux!|");
		}
		
		################################################
		#IP check. Reads from file. If file does not exist, any IP will be accepted.
		################################################
		$str=$scriptdir.".ipmask";
		my $ip_ok="no";
		open(IPFILE,$str) or $ip_ok="yes";
		if($ip_ok eq "no")
		{
			$char="";
			$str="";
			while(not eof(IPFILE))
			{
				
				read(IPFILE,$char,1);
				if(ord($char)!=10)
				{
					$str=$str.$char;
				}
				else
				{
					if(($str eq substr($ip,0,length($str))) and length($str)>1)
					{
						#IP ok.
						$ip_ok="yes";
					}
					$str="";
				}
			}
			close(IPFILE);
		}
		if(($ip_ok eq "no") and ($uinfo[5]==0))
		{
			dchub::send_to_named_user($evt_array{"nickname"},"<Hub-Security> $you_have_got_to_have_a_specifik_ip|");
			dchub::send_pchat_msg("Hub-Log",$hubmaster,$evt_array{"nickname"}." IP:".$ip.$disconnected_because_wrong_ip."|");
			dchub::disconnect($evt_array{"nickname"});
			return;
		}
		
		################################################
		#Nick check.
		################################################
		if($uinfo[5]==0)
		{
			$str=$scriptdir.".prefixchk";
			my $nick_ok="no";
			open(PREFIXFILE, $str) or $nick_ok="yes";
			if($nick_ok eq "no")
			{
				$char="";
				while(not eof(PREFIXFILE))
				{
					read(PREFIXFILE,$char,1);
					if(ord($char)!=10)
					{
						$str=$str.$char;
					}
					else
					{
						if(($str eq substr($evt_array{"nickname"},0,length($str))) and length($str)>1)
						{
							#Nick ok.
							$nick_ok="yes";
						}
						$str="";
					}
				}
				close(PREFIXFILE);
			}
			if($nick_ok eq "no")
			{
				my $str=$scriptdir.".namebot";
				open(NAMEFILE, $str) or $str="err"; 
				if($str ne "err")
				{
					my $char="";
					while(not eof(NAMEFILE))
					{
						read(NAMEFILE,$char,1);
						if(ord($char)!=10)
						{
							$str=$str.$char;
						}
						else
						{
							$str=$str.chr("13").chr("10");
						}
					}
					close(NAMEFILE);
					dchub::send_pchat_msg("Name-Bot",$evt_array{"nickname"},$str."|");
				}
				dchub::send_pchat_msg("Hub-Log",$hubmaster,$evt_array{"nickname"}." IP:".$ip.$disconnected_because_wrong_prefix."|");
				dchub::disconnect($evt_array{"nickname"});
				return;
			}
		}
			
		################################################
		#Minimun share check.
		################################################
		my $min_share=dchub::db_get("MIN_SHARE");
		if(defined($min_share))
		{
			if($uinfo[1]<$min_share)
			{
				$str=$you_must_share_more_than;

				if($min_share<1024)
				{
					$str.="$min_share ";
				}
				elsif($min_share<(1024*1024))
				{
					$str.=($min_share/1024)." K";
				}
				elsif($min_share<(1024*1024*1024))
				{
					$str.=($min_share/(1024*1024))." M";
				}
				else
				{
					$str.=($min_share/(1024*1024*1024))." G";
				}
	
				$str.=$bytes_Add_some_files_and_come_back_later;

				if($uinfo[5]==0)
				{
					dchub::send_to_named_user($evt_array{"nickname"},"<Hub-Security> $str|");
					dchub::disconnect($evt_array{"nickname"});
					return;
				}
				else
				{
					if($duration<4)
					{
						dchub::send_to_named_user($evt_array{"nickname"},"<Hub-Security> $str $hopefully_you_are_an_op |");
					}
				}
			}
		}
		
		################################################
		#dc++ number of hub and slot's shared checker
		#This part must be rewritten, the included script does not work satisfying!!! /Daniel Marmander
		#For now, it will be excluded.
		################################################
		
		################################################
		#Logg.
		################################################
		if($duration<4)
		{
			dchub::send_pchat_msg("Hub-Log",$hubmaster,$evt_array{"nickname"}." IP:".$ip."".$logged_on."|");
			if(dchub::db_get("HUB_LOG") eq "TRUE")
			{
				#Log 
				$str=$logdir."hub.log";
				open LOGFILE, ">>$str" or die "Can't open $str: $!";#This is not error tolerant. Fix.
				print LOGFILE $evt_array{"nickname"}." IP:".$ip."".$logged_on."\n";
				close(LOGFILE);
				
				#Log number of users:
				my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);  
				my $users=scalar(split(/\$/,dchub::userlist()));
				$str=$logdir.$year.$mon.$mday.".log";
				open LOGFILE, ">>$str" or die "Can't open $str: $!";#This is not error tolerant. Fix.
				$str=$hour." ".$min." ".$users.";";
				print LOGFILE $str."\n";
				close(LOGFILE);
			}
		}
	}
   print "#########################################################\n";
}

