#! /usr/bin/perl
# this variable is used to avoid infinite recursive calls of AUTOLOAD
# if catching==1, the error occurs inside AUTOLOAD function itself
use strict;

my $catching=0;

my $script_dir="";

my @all_autoloaded_perl_handlers;

#***********************************************************************
# initialize some variables just after the interpreter has been loaded 
#***********************************************************************
# parameter: 1 string: the script dir
sub dchub_perl_init {
	$script_dir=$_[0];
	my $autostart_file="$script_dir/AUTOSTART";
	if (open(AUTO,$autostart_file)) {
		while (<AUTO>) {
			chomp;
			if (/\.pl$/) {
				my $filename="$script_dir/$_";
				if (open(SCRIPT,$filename)) {
					local($/) = undef;
					my $eval=<SCRIPT>;
					close(SCRIPT);
					eval $eval;
					print "Perl script: $_ loaded\n";
				}
			}
		}
	}
}

#***********************************************************************
# Catch all invalid method calls 
# the function tries to load a file named "$script_dir/$event_name.pl" 
# if the file successfully loads, it tries to call the function one 
# more time.
#***********************************************************************
sub AUTOLOAD
{
	my ($self, @args) = @_;
	# my ($class)		= ref $self || $self;

	if($catching==0)
	{
		local *FH;
		our ($AUTOLOAD);

		$catching=1;
		# try to load a package containing the given handler
		my $funcname=$AUTOLOAD;
		$funcname=~ s/^.*:://;
		my $filename="$script_dir/$funcname.pl";
		
		unless (open(FH, $filename))
		{
			print STDERR "WARNING: Can't open $filename: $!\n";
			$catching=0;
			return;
		}
		local($/) = undef;
		my $eval= <FH>;
		close FH;

		eval $eval;
		dir $@ if $@;
		
		push @all_autoloaded_perl_handlers,$AUTOLOAD;
		no strict 'refs';
		&$funcname(@args);
		$catching=0;
	}

	return;
}

#***********************************************************************
# undefined all autoloaded perl handlers 
#***********************************************************************
sub dchub_perl_clear_autoloaded_handlers
{
	no strict 'refs';
	my $sub;
	while ($sub=shift(@all_autoloaded_perl_handlers)) {
		undef *$sub;
	}
}

#******************************************************************************
#******************************************************************************
#  Event handler entry point
#******************************************************************************
#******************************************************************************
# parameter: 1 hash table (evt_array)
sub dchub_perl_handler
{
  our %evt_array;
  *evt_array = $_[0];		# evt_array is a hash table (%) containing all the parameters 
							# the event array contains at least the following entries 
							# "event": the name of the event 
							# "nickname": the nickname of the user generating this event 
							# "argc": the number of optionnal arguments 
							# if optionnal arguments exist, they are named "0", "1", "2", "3, ... 
  # begin debug code: debug print
  #my $num;
  #
  #printf "event	 = '".$evt_array{"event"}."'\n";
  #printf "nickname= '".$evt_array{"nickname"}."'\n";
  #printf "argc	 = '".$evt_array{"argc"}."'\n";
  #
  #$num=0;
  #while($num<$evt_array{"argc"})
  #{
  #print "key $num = '".$evt_array{$num}."'\n";
  #$num++;
  #}
  # end debug code: debug print
	
  my $hdl_name="dchub_".$evt_array{"event"}."_handler";

  # call the handle which must deal with this event
  no strict 'refs';
  &$hdl_name;
}
